@@ -16,15 +16,15 @@ import ton.tonlibjson
 from .utils import *
 
 
-async def _get_client(ls: int, timeout: int):
+async def _get_client(ls: int, timeout: int, cdll_path: str = None):
     TonlibClient.enable_unaudited_binaries()
     client = TonlibClient(ls_index=ls, default_timeout=timeout)
-    await client.init_tonlib()
+    await client.init_tonlib(cdll_path=cdll_path)
     return client
 
 
-def get_client(ls: int = 0, timeout: int = 30) -> TonlibClient:
-    client = asyncio.get_event_loop().run_until_complete(_get_client(ls, timeout))
+def get_client(ls: int = 0, timeout: int = 30, cdll_path: str = None) -> TonlibClient:
+    client = asyncio.get_event_loop().run_until_complete(_get_client(ls, timeout, cdll_path))
     return client
 
 
@@ -178,12 +178,12 @@ async def _get_all_wallet_transactions(client: TonlibClient, addr: str, limit: i
         if len(tr['out_msgs']):
             tr_result['type'] = 'out'
             tr_result['from'] = Address(addr).to_string(True, True, True)
-            tr_result['to'] = Address(tr['out_msgs'][0]['destination']['account_address']).to_string(True, True, True)
+            tr_result['to'] = Address(tr['out_msgs'][0]['destination']['account_address']).to_string(True, True, True) if tr['out_msgs'][0]['destination']['account_address'] else ''
             tr_result['value'] = int(tr['out_msgs'][0]['value'])
             tr_result['message'] = base64.b64decode(tr['out_msgs'][0]['msg_data']['text']).decode('utf-8') if tr['out_msgs'][0]['msg_data']['@type'] == 'msg.dataText' else ''
         else:
             tr_result['type'] = 'in'
-            tr_result['from'] = Address(tr['in_msg']['source']['account_address']).to_string(True, True, True)
+            tr_result['from'] = Address(tr['in_msg']['source']['account_address']).to_string(True, True, True) if tr['in_msg']['source']['account_address'] else ''
             tr_result['to'] = Address(addr).to_string(is_user_friendly=True)
             tr_result['value'] = int(tr['in_msg']['value'])
             tr_result['message'] = base64.b64decode(tr['in_msg']['msg_data']['text']).decode('utf-8') if tr['in_msg']['msg_data']['@type'] == 'msg.dataText' else ''
@@ -270,7 +270,7 @@ async def _get_collection_items(client: TonlibClient, addr: str):
     return result
 
 
-def transfer_ton(mnemonic: list, address: str, value: float, comment: str = '', wallet_version: str = 'v4r2', client: TonlibClient = None):
+def transfer_ton(mnemonic: list, address: str, value: float, comment: str = '', wallet_version: str = 'v4r2', client: TonlibClient = None, send_mode=3):
     """
     :param mnemonic: seed phrase of owner wallet
     :param address: address to send Tons
@@ -282,19 +282,20 @@ def transfer_ton(mnemonic: list, address: str, value: float, comment: str = '',
     if not client:
         client = get_client(2)
 
-    return asyncio.get_event_loop().run_until_complete(_transfer_ton(client, mnemonic, address, value, comment, wallet_version))
+    result = asyncio.get_event_loop().run_until_complete(_transfer_ton(client, mnemonic, address, value, comment, wallet_version, send_mode))
+    close_client(client)
+    return result
 
 
-async def _transfer_ton(client: TonlibClient, mnemonic: list, address: str, value: float, comment: str, wallet_version: str):
+async def _transfer_ton(client: TonlibClient, mnemonic: list, address: str, value: float, comment: str, wallet_version: str, send_mode=3):
     mnemonics, _pub_k, _priv_k, wallet = Wallets.from_mnemonics(mnemonic, WalletVersionEnum(wallet_version), 0)
     account = await client.find_account(wallet.address.to_string())
     seqno = await account.seqno()
     query = wallet.create_transfer_message(to_addr=address,
                                           amount=to_nano(value, 'ton'),
-                                          seqno=seqno, payload=comment)
+                                          seqno=seqno, payload=comment, send_mode=send_mode)
     boc = query["message"].to_boc(False)
     resp = await client.send_boc(boc)
-    await client.tonlib_wrapper.close()
     return resp
 
 
